module Sungemm
  # Finds the n-th fibonachi number.
  # @param n [Fixnum] element number in the fibonachi sequence.
  # @return [Fixnum] N-th fibo number.
  def self.fibo_finder(n)
    if (0..1).include?(n)
      n
    else
      fibo_finder(n - 2) + fibo_finder(n - 1)
    end
  end

  def self.longest_palindrome(str)
    len = str.length
    while len > 0
      offset = str.length - len
      while offset > 0
        substr = str[offset, len]
        return substr if substr == substr.reverse
        offset -= 1
      end
      len -= 1
    end
    ""
  end

  def self.count(array)
    h = Hash.new(0)
    array.each { |el| h[el] += 1 }
    h
  end

  def self.even_sum(arr)
    arr.each_with_object([]) do |item, obj|
      if !item.nil? && item.length.even?
        obj << item.reverse
      end
    end
  end

  def self.join_params(link, url_parameters)
    link + "?" + url_parameters.map { |(k, v)| "#{k}=#{v}" }.join("&")
  end

  def self.pretty_date(date)
    month_strings = %w(January February March April May June July August
                       Sepetember October November December)
    invalid_date = false
    case
    when m = /\A(\d{2})-(\d{2})-(\d{4})\z/.match(date)
      year = m[3]
      month_int = m[1].to_i - 1
      month = month_strings[month_int][0, 3]
      day = m[2]
    when m = /\A(\w+)[ ](\d{1,2}),[ ](\d{4})\z/.match(date)
      month_match = m[1]
      if !month_strings.include?(month_match)
        invalid_date = true
      else
        month = month_match[0, 3]
        year = m[3]
        day = m[2]
      end
    when m = /\A(\d{4})-(\d{2})-(\d{2})\z/.match(date)
      year = m[1]
      month_int = m[2].to_i - 1
      month = month_strings[month_int][0, 3]
      day = m[3]
    when m = %r{\A(\d{1,2})/(\d{2})/(\d{4})\z}.match(date)
      year = m[3]
      month_int = m[1].to_i - 1
      month = month_strings[month_int][0, 3]
      day = m[2]
    else
      invalid_date = true
    end
    raise "Invalid date format" if invalid_date
    "#{month} #{day}, #{year}"
  end

  def self.random_boolean
    rand(2) == 1
  end

  def self.reverse_each_word(sentence)
    sentence.split(" ").map { |x| x.reverse }.join(" ")
  end

  def self.separate_with_comma(number)
    str = number.to_s
    if str.length > 3
      str.gsub!(/(\d)(?=(\d\d\d)+(?!\d))/, "\\1,")
    else
      str
    end
  end

  def self.simple_sub(str, pattern, replace)
    str.gsub(pattern, pattern => replace)
  end

  def self.key_for_min_value(hash)
    hash.key(hash.values.min)
  end
end
